@@ -12,6 +12,9 @@ import (
 )
 
 func Test(t *testing.T) {
+	modernize.EnableSlicesDelete = true
+	modernize.EnableAppendClipped = true
+
 	analysistest.RunWithSuggestedFixes(t, analysistest.TestData(), modernize.Analyzer,
 		"appendclipped",
 		"bloop",
