@@ -396,6 +396,7 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 	}
 
 	// Compute size information for types,
+	// including allocator size class,
 	// and (size, offset) for struct fields.
 	//
 	// Also, if a struct type's field ordering is significantly
@@ -430,7 +431,7 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 
 		path := pathEnclosingObjNode(pgf.File, pos)
 
-		// Build string of form "size=... (X% wasted), offset=...".
+		// Build string of form "size=... (X% wasted), class=..., offset=...".
 		size, wasted, offset := computeSizeOffsetInfo(pkg, path, obj)
 		var buf strings.Builder
 		if size >= 0 {
@@ -438,6 +439,11 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 			if wasted >= 20 { // >=20% wasted
 				fmt.Fprintf(&buf, " (%d%% wasted)", wasted)
 			}
+
+			// Include allocator size class, if larger.
+			if class := sizeClass(size); class > size {
+				fmt.Fprintf(&buf, ", class=%s", format(class))
+			}
 		}
 		if offset >= 0 {
 			if buf.Len() > 0 {
@@ -495,7 +501,7 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 					types.TypeString(f.field.Type(), qual),
 					f.path)
 			}
-			w.Flush()
+			w.Flush() // ignore error
 			b.WriteByte('\n')
 			fields = b.String()
 		}
@@ -1040,7 +1046,7 @@ func hoverReturnStatement(pgf *parsego.File, path []ast.Node, ret *ast.ReturnStm
 		if i > 0 {
 			buf.WriteString(", ")
 		}
-		cfg.Fprint(&buf, fset, field.Type)
+		cfg.Fprint(&buf, fset, field.Type) // ignore error
 	}
 	buf.WriteByte(')')
 	return rng, &hoverResult{
@@ -1776,3 +1782,14 @@ func computeSizeOffsetInfo(pkg *cache.Package, path []ast.Node, obj types.Object
 
 	return
 }
+
+// sizeClass reports the size class for a struct of the specified size, or -1 if unknown.f
+// See GOROOT/src/runtime/msize.go for details.
+func sizeClass(size int64) int64 {
+	if size > 1<<16 {
+		return -1 // avoid allocation
+	}
+	// We assume that bytes.Clone doesn't trim,
+	// and reports the underlying size class; see TestSizeClass.
+	return int64(cap(bytes.Clone(make([]byte, size))))
+}
