@@ -121,7 +121,7 @@ func (s *server) DidChangeConfiguration(ctx context.Context, _ *protocol.DidChan
 		}
 		newFolders = append(newFolders, newFolder)
 	}
-	s.session.UpdateFolders(ctx, newFolders)
+	s.session.UpdateFolders(ctx, newFolders) // ignore error
 
 	// The view set may have been updated above.
 	viewsToDiagnose := make(map[*cache.View][]protocol.DocumentURI)
@@ -149,7 +149,7 @@ func (s *server) DidCreateFiles(ctx context.Context, params *protocol.CreateFile
 	var allChanges []protocol.DocumentChange
 	for _, createdFile := range params.Files {
 		uri := protocol.DocumentURI(createdFile.URI)
-		fh, snapshot, release, err := s.fileOf(ctx, uri)
+		fh, snapshot, release, err := s.session.FileOf(ctx, uri)
 		if err != nil {
 			event.Error(ctx, "fail to call fileOf", err)
 			continue
